@@ -13,9 +13,10 @@ module Copilot.Core.Operators
   where
 
 -- External imports
-import Data.Bits    (Bits)
-import Data.Word    (Word32)
-import GHC.TypeLits (KnownSymbol)
+import Data.Bits     (Bits)
+import Data.Typeable (Typeable)
+import Data.Word     (Word32)
+import GHC.TypeLits  (KnownSymbol)
 
 -- Internal imports
 import Copilot.Core.Type       (Field (..), Type (..))
@@ -96,7 +97,15 @@ data Op2 a b c where
   Index    :: Type (Array n t) -> Op2 (Array n t) Word32 t
               -- ^ Array access/projection of an array element.
 
+  -- Struct operator.
+  UpdateField :: (Typeable b, KnownSymbol s, Show b)
+              => Type a -> Type b -> (a -> Field s b) -> Op2 a b a
+              -- ^ Update a field of a struct.
+
 -- | Ternary operators.
 data Op3 a b c d where
   -- Conditional operator.
   Mux :: Type a -> Op3 Bool a a a
+  -- Array operator.
+  UpdateArray :: Type (Array n t) -> Op3 (Array n t) Word32 t (Array n t)
+           -- ^ Update an element of an array.
