@@ -3,6 +3,7 @@
 {-# LANGUAGE Safe                #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeFamilies        #-}
+{-# LANGUAGE TypeOperators       #-}
 
 -- |
 -- Copyright: (c) 2011 National Institute of Aerospace / Galois, Inc.
@@ -14,12 +15,13 @@ module Copilot.Core.Type.Array
     ( Array
     , array
     , arrayElems
+    , arrayUpdate
     )
   where
 
 -- External imports
 import Data.Proxy   (Proxy (..))
-import GHC.TypeLits (KnownNat, Nat, natVal)
+import GHC.TypeLits (KnownNat, Nat, natVal, type(-))
 
 -- | Implementation of an array that uses type literals to store length.
 data Array (n :: Nat) t where
@@ -42,3 +44,21 @@ array xs | datalen == typelen = Array xs
 -- | Return the elements of an array.
 arrayElems :: Array n a -> [a]
 arrayElems (Array xs) = xs
+
+-- | Update element of array to given element.
+--
+-- PRE: the second argument denotes a valid index in the array.
+arrayUpdate :: Array n a -> Int -> a -> Array n a
+arrayUpdate (Array []) _ _ = error errMsg
+  where
+    errMsg = "copilot-core: arrayUpdate: Attempt to update empty array"
+
+arrayUpdate (Array (x:xs)) 0 y = Array (y:xs)
+
+arrayUpdate (Array (x:xs)) n y =
+    arrayAppend x (arrayUpdate (Array xs) (n - 1) y)
+  where
+    -- | Append to an array while preserving length information at the type
+    -- level.
+    arrayAppend :: a -> Array (n - 1) a -> Array n a
+    arrayAppend x (Array xs) = Array (x:xs)
